@@ -1,5 +1,5 @@
-# WordNet::Similarity::lch.pm version 0.07
-# (Updated 3/07/2004 -- Jason)
+# WordNet::Similarity::lch.pm version 1.01
+# (Last update $Id: lch.pm,v 1.22 2005/12/11 22:37:02 sidz1979 Exp $)
 #
 # Semantic Similarity Measure package implementing the measure
 # described by Leacock and Chodorow (1998).
@@ -47,11 +47,11 @@ of this scaled value.
 use strict;
 
 use Exporter;
-use WordNet::Similarity::LCSFinder;
+use WordNet::Similarity::DepthFinder;
 
-our @ISA = qw/WordNet::Similarity::LCSFinder/;
+our @ISA = qw/WordNet::Similarity::DepthFinder/;
 
-our $VERSION = '0.07';
+our $VERSION = '1.01';
 
 sub setPosList
 {
@@ -312,7 +312,7 @@ perl(1), WordNet::Similarity(3), WordNet::QueryData(3)
 
 http://www.cs.utah.edu/~sidd
 
-http://www.cogsci.princeton.edu/~wn
+http://wordnet.princeton.edu
 
 http://www.ai.mit.edu/~jrennie/WordNet
 
@@ -320,15 +320,15 @@ http://groups.yahoo.com/group/wn-similarity
 
 =head1 AUTHORS
 
+ Ted Pedersen, University of Minnesota Duluth
+ tpederse at d.umn.edu
+
  Siddharth Patwardhan, University of Utah, Salt Lake City
  sidd at cs.utah.edu
 
  Jason Michelizzi, University of Minnesota Duluth
  mich0212 at d.umn.edu
 
- Ted Pedersen, University of Minnesota Duluth
- tpederse at d.umn.edu
-
 =head1 BUGS
 
 None.
@@ -338,8 +338,7 @@ e-mail tpederse at d.umn.edu.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2003-2004, Siddharth Patwardhan, Jason Michelizzi, and Ted
-Pedersen
+Copyright (c) 2005, Ted Pedersen, Siddharth Patwardhan and Jason Michelizzi
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
