@@ -1,16 +1,18 @@
-# get_wn_info.pm version 0.07
-# (Updated 10/11/2003 -- Jason)
+# get_wn_info.pm version 1.01
+# (Last updated $Id: get_wn_info.pm,v 1.10 2005/12/11 22:37:02 sidz1979 Exp $)
 #
 # Package used by WordNet::Similarity::lesk module that
 # computes semantic relatedness of word senses in WordNet
 # using gloss overlaps.
 #
-# Copyright (c) 2003,
-# Satanjeev Banerjee, Carnegie Mellon University, Pittsburgh
-# banerjee+ at cs.cmu.edu
+# Copyright (c) 2005,
+#
 # Ted Pedersen, University of Minnesota, Duluth
 # tpederse at d.umn.edu
 #
+# Satanjeev Banerjee, Carnegie Mellon University, Pittsburgh
+# banerjee+ at cs.cmu.edu
+#
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
@@ -27,6 +29,8 @@
 # The Free Software Foundation, Inc., 
 # 59 Temple Place - Suite 330, 
 # Boston, MA  02111-1307, USA.
+#
+# ------------------------------------------------------------------
 
 package get_wn_info;
 
@@ -43,7 +47,7 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 
 @EXPORT = ();
 
-$VERSION = '0.07';
+$VERSION = '1.01';
 
 # function to set up the wordnet object and the various boundary indices
 sub new
@@ -62,7 +66,7 @@ sub new
     $self->{'wn'} = $wn;
 
     # check WordNet::QueryData version
-    $wn->VERSION(1.30);
+    $wn->VERSION(1.39);
 
     # check if stemming called for 
     $stemmingReqd = shift;
@@ -84,6 +88,8 @@ sub new
     return $self;
 }
 
+# NOTE: Thanks to Wybo Wiersma for contributing optimizations
+#       in the following code.
 
 # function to take a set of synsets and to return their
 # hypernyms. both input and output will be arrays of fully qualified
@@ -92,41 +98,37 @@ sub hype
 {
     my $self = shift;
     my $wn = $self->{'wn'};
-    my @synsets = @_;
-    my %hypernymHash = ();
+    my ($synsetsh, $outprep) = @_;
 
     # check if this is a request for the input-output types of this
     # function
-    if ($#synsets == 0 && $synsets[0] eq '0')
-    {
-	# ah!
-	return(1, 1);
-    }
+    return(1, 1) if(defined($outprep));
 
-    my $i = 0;
-    for (; $i <= $#synsets; $i++)
+    my %newsynsetsh;
+    foreach my $syns (keys %{$synsetsh})
     {
 	# check if in word-pos-sense format
-	if ($synsets[$i] !~ /\#\w\#\d+/)
+        # TODO: Return error code instead of "exit"
+	if($syns !~ /\#\w\#\d+/)
 	{
-	    print STDERR "$synsets[$i] is not in WORD\#POS\#SENSE format!\n";
+	    print STDERR "$syns is not in WORD\#POS\#SENSE format!\n";
 	    exit;
 	}
 	
 	# get the hypernyms
-	my @hypernyms = $wn->querySense($synsets[$i], "hype");
+	my @hypernyms = $wn->querySense($syns, "hypes");
 	
 	# put the hypernyms in a hash. this way we will avoid multiple
 	# copies of the same hypernym
 	my $temp;
 	foreach $temp (@hypernyms)
 	{
-	    $hypernymHash{$temp} = 1;
+	    $newsynsetsh{$temp} = 1;
 	}
     }
     
-    # return the hypernyms in an array 
-    return(sort keys %hypernymHash);
+    # return the hypernyms in an hash ref 
+    return(\%newsynsetsh);
 }
 
 # function to take a set of synsets and to return their
@@ -136,29 +138,25 @@ sub hypo
 {
     my $self = shift;
     my $wn = $self->{'wn'};
-    my @synsets = @_;
-    my %hyponymHash = ();
+    my ($synsetsh, $outprep) = @_;
     
     # check if this is a request for the input-output types of this
     # function
-    if ($#synsets == 0 && $synsets[0] eq '0')
-    {
-	# ah!
-	return(1, 1);
-    }
+    return(1, 1) if(defined($outprep));
     
-    my $i = 0;
-    for (; $i <= $#synsets; $i++)
+    my %hyponymHash;
+    foreach my $syns (keys %{$synsetsh})
     {
 	# check if in word-pos-sense format
-	if ($synsets[$i] !~ /\#\w\#\d+/)
+        # TODO: Replace error message and exit with return error code.
+	if($syns !~ /\#\w\#\d+/)
 	{
-	    print STDERR "$synsets[$i] is not in WORD\#POS\#SENSE format!\n";
+	    print STDERR "$syns is not in WORD\#POS\#SENSE format!\n";
 	    exit;
 	}
 	
 	# get the hyponyms
-	my @hyponyms = $wn->querySense($synsets[$i], "hypo");
+	my @hyponyms = $wn->querySense($syns, "hypos");
 	
 	# put the hyponyms in a hash. this way we will avoid multiple
 	# copies of the same hyponym
@@ -169,8 +167,8 @@ sub hypo
 	}
     }
     
-    # return the hyponyms in an array 
-    return(sort keys %hyponymHash);
+    # return the hyponyms in an hash ref
+    return(\%hyponymHash);
 }
 
 # function to take a set of synsets and to return their
@@ -180,32 +178,28 @@ sub holo
 {
     my $self = shift;
     my $wn = $self->{'wn'};
-    my @synsets = @_;
+    my ($synsetsh, $outprep) = @_;
     my %holonymHash = ();
     
     # check if this is a request for the input-output types of this
     # function
-    if ($#synsets == 0 && $synsets[0] eq '0')
-    {
-	# ah!
-	return(1, 1);
-    }
+    return(1, 1) if(defined($outprep));
 
-    my $i = 0;
-    for (; $i <= $#synsets; $i++)
+    foreach my $syns (keys %{$synsetsh})
     {
 	# check if in word-pos-sense format
-	if ($synsets[$i] !~ /\#\w\#\d+/)
+        # TODO: Replace error message and exit with return error code.
+	if($syns !~ /\#\w\#\d+/)
 	{
-	    print STDERR "$synsets[$i] is not in WORD\#POS\#SENSE format!\n";
+	    print STDERR "$syns is not in WORD\#POS\#SENSE format!\n";
 	    exit;
 	}
 	
 	# get the holonyms
-	my @holonyms = $wn->querySense($synsets[$i], "holo");
+	my @holonyms = $wn->querySense($syns, "holo");
 	
 	# put the holonyms in a hash. this way we will avoid multiple
-		# copies of the same holonym
+        # copies of the same holonym
 	my $temp;
 	foreach $temp (@holonyms)
 	{
@@ -213,8 +207,8 @@ sub holo
 	}
     }
     
-    # return the holonyms in an array 
-    return(sort keys %holonymHash);
+    # return the holonyms in an hash ref 
+    return(\%holonymHash);
 }
 
 # function to take a set of synsets and to return their
@@ -224,29 +218,25 @@ sub mero
 {
     my $self = shift;
     my $wn = $self->{'wn'};
-    my @synsets = @_;
+    my ($synsetsh, $outprep) = @_;
     my %meronymHash = ();
     
     # check if this is a request for the input-output types of this
     # function
-    if ($#synsets == 0 && $synsets[0] eq '0')
-    {
-	# ah!
-	return(1, 1);
-    }
+    return (1, 1) if(defined($outprep));
     
-    my $i = 0;
-    for (; $i <= $#synsets; $i++)
+    foreach my $syns (keys %{$synsetsh})
     {
 	# check if in word-pos-sense format
-	if ($synsets[$i] !~ /\#\w\#\d+/)
+        # TODO: Replace error message and exit with return error code.
+	if($syns !~ /\#\w\#\d+/)
 	{
-	    print STDERR "$synsets[$i] is not in WORD\#POS\#SENSE format!\n";
+	    print STDERR "$syns is not in WORD\#POS\#SENSE format!\n";
 	    exit;
 	}
 	
 	# get the meronyms
-	my @meronyms = $wn->querySense($synsets[$i], "mero");
+	my @meronyms = $wn->querySense($syns, "mero");
 	
 	# put the meronyms in a hash. this way we will avoid multiple
 	# copies of the same meronym
@@ -257,8 +247,8 @@ sub mero
 	}
     }
     
-    # return the meronyms in an array 
-    return(sort keys %meronymHash);
+    # return the meronyms in an hash ref 
+    return(\%meronymHash);
 }
 
 # function to take a set of synsets and to return their
@@ -268,29 +258,24 @@ sub attr
 {
     my $self = shift;
     my $wn = $self->{'wn'};
-    my @synsets = @_;
+    my ($synsetsh, $outprep) = @_;
     my %attrHash = ();
     
     # check if this is a request for the input-output types of this
     # function
-    if ($#synsets == 0 && $synsets[0] eq '0')
-    {
-	# ah!
-	return(1, 1);
-    }
+    return (1, 1) if(defined($outprep));
     
-    my $i = 0;
-    for (; $i <= $#synsets; $i++)
+    foreach my $syns (keys %{$synsetsh})
     {
 	# check if in word-pos-sense format
-	if ($synsets[$i] !~ /\#\w\#\d+/)
+	if($syns !~ /\#\w\#\d+/)
 	{
-	    print STDERR "$synsets[$i] is not in WORD\#POS\#SENSE format!\n";
+	    print STDERR "$syns is not in WORD\#POS\#SENSE format!\n";
 	    exit;
 	}
 	
 	# get the attrs
-	my @attrs = $wn->querySense($synsets[$i], "attr");
+	my @attrs = $wn->querySense($syns, "attr");
 	
 	# put the attrs in a hash. this way we will avoid multiple
 	# copies of the same attr
@@ -301,8 +286,8 @@ sub attr
 	}
     }
     
-    # return the attrs in an array 
-    return(sort keys %attrHash);
+    # return the attrs in an hash ref 
+    return(\%attrHash);
 }
 
 # function to take a set of synsets and to return their also-see
@@ -312,29 +297,24 @@ sub also
 {
     my $self = shift;
     my $wn = $self->{'wn'};
-    my @synsets = @_;
+    my ($synsetsh, $outprep) = @_;
     my %alsoSeeHash = ();
     
     # check if this is a request for the input-output types of this
     # function
-    if ($#synsets == 0 && $synsets[0] eq '0')
-    {
-	# ah!
-	return(1, 1);
-    }
-    
-    my $i = 0;
-    for (; $i <= $#synsets; $i++)
+    return (1, 1) if(defined($outprep));
+
+    foreach my $syns (keys %{$synsetsh})
     {
 	# check if in word-pos-sense format
-	if ($synsets[$i] !~ /\#\w\#\d+/)
+	if($syns !~ /\#\w\#\d+/)
 	{
-	    print STDERR "$synsets[$i] is not in WORD\#POS\#SENSE format!\n";
+	    print STDERR "$syns is not in WORD\#POS\#SENSE format!\n";
 	    exit;
 	}
 	
 	# get the also see synsets 
-	my @alsoSees = $wn->queryWord($synsets[$i], "also");
+	my @alsoSees = $wn->queryWord($syns, "also");
 	
 	# put the synsets in a hash. this way we will avoid multiple
 	# copies of the same synset
@@ -345,8 +325,8 @@ sub also
 	}
     }
     
-    # return the synsets in an array 
-    return(sort keys %alsoSeeHash);
+    # return the synsets in an hash ref 
+    return(\%alsoSeeHash);
 }
 
 # function to take a set of words and to return their derived forms. 
@@ -356,25 +336,27 @@ sub deri
 {
     my $self = shift;
     my $wn = $self->{wn};
-    my @words = @_;
+    my ($wordsh, $outprep) = @_;
     my %deriHash = ();
 
-    if ($#words == 0 and $words[0] eq '0') {
-	return (1, 1);
-    }
+    return (1, 1) if(defined($outprep));
 
-    for my $i (0..$#words) {
-	unless ($words[$i] =~ m/\#\w+\#\d+/) {
-	    print STDERR "$words[$i] is not in WORD#POS#SENSE format!\n";
+    foreach my $word (keys %{$wordsh}) 
+    {
+        # TODO: Replace error message and exit with return error code.
+	if($word !~ m/\#\w+\#\d+/) 
+        {
+	    print STDERR "$word is not in WORD#POS#SENSE format!\n";
 	    exit 1;
 	}
-	my @deris = $wn->queryWord ($words[$i], "deri");
+	my @deris = $wn->queryWord($word, "deri");
 
-	foreach my $temp (@words) {
+	foreach my $temp (@deris) 
+        {
 	    $deriHash{$temp} = 1;
 	}
     }
-    return sort (keys %deriHash);
+    return(\%deriHash);
 }
 
 # function to take a set of synsets and to return their domain
@@ -384,26 +366,27 @@ sub domn
 {
     my $self = shift;
     my $wn = $self->{wn};
-    my @words = @_;
+    my ($wordsh, $outprep) = @_;
     my %domnHash = ();
 
-    if ($#words == 0 and $words[0] eq '0') {
-	return (1, 1);
-    }
+    return(1, 1) if(defined($outprep));
 
-    for my $i (0..$#words) {
-	unless ($words[$i] =~ m/\#\w+\#\d+/) {
-	    print STDERR "$words[$i] is not in WORD#POS#SENSE format!\n";
+    foreach my $word (keys %{$wordsh})
+    {
+        # TODO: Replace error message and exit with return error code.
+	if($word !~ m/\#\w+\#\d+/)
+        {
+	    print STDERR "$word is not in WORD#POS#SENSE format!\n";
 	    exit 1;
 	}
-	my @domns = $wn->queryWord ($words[$i], "deri");
+	my @domns = $wn->queryWord($word, "domn");
 
-	foreach my $temp (@words) {
+	foreach my $temp (@domns) 
+        {
 	    $domnHash{$temp} = 1;
 	}
     }
-    return sort (keys %domnHash);
-
+    return (\%domnHash);
 }
 
 # function to take a set of synsets and to return their domain term
@@ -413,25 +396,27 @@ sub domt
 {
     my $self = shift;
     my $wn = $self->{wn};
-    my @words = @_;
+    my ($wordsh, $outprep) = @_;
     my %domtHash = ();
 
-    if ($#words == 0 and $words[0] eq '0') {
-	return (1, 1);
-    }
+    return (1, 1) if(defined($outprep));
 
-    for my $i (0..$#words) {
-	unless ($words[$i] =~ m/\#\w+\#\d+/) {
-	    print STDERR "$words[$i] is not in WORD#POS#SENSE format!\n";
+    foreach my $word (keys %{$wordsh})
+    {
+        # TODO: Replace error message and exit with return error code.
+	if($word != m/\#\w+\#\d+/)
+        {
+	    print STDERR "$word is not in WORD#POS#SENSE format!\n";
 	    exit 1;
 	}
-	my @domts = $wn->queryWord ($words[$i], "deri");
+	my @domts = $wn->queryWord ($word, "domt");
 
-	foreach my $temp (@words) {
+	foreach my $temp (@domts)
+        {
 	    $domtHash{$temp} = 1;
 	}
     }
-    return sort (keys %domtHash);
+    return (\%domtHash);
 
 }
 
@@ -442,29 +427,25 @@ sub sim
 {
     my $self = shift;
     my $wn = $self->{'wn'};
-    my @synsets = @_;
+    my ($synsetsh, $outprep) = @_;
     my %simHash = ();
     
     # check if this is a request for the input-output types of this
     # function
-    if ($#synsets == 0 && $synsets[0] eq '0')
-    {
-	# ah!
-	return(1, 1);
-    }
-    
-    my $i = 0;
-    for (; $i <= $#synsets; $i++)
+    return (1, 1) if(defined($outprep));
+
+    foreach my $syns (keys %{$synsetsh})
     {
 	# check if in word-pos-sense format
-	if ($synsets[$i] !~ /\#\w\#\d+/)
+        # TODO: Replace error message and exit with return error code.
+	if($syns !~ /\#\w\#\d+/)
 	{
-	    print STDERR "$synsets[$i] is not in WORD\#POS\#SENSE format!\n";
+	    print STDERR "$syns is not in WORD\#POS\#SENSE format!\n";
 	    exit;
 	}
 	
 	# get the sim synsets 
-	my @sims = $wn->querySense($synsets[$i], "sim");
+	my @sims = $wn->querySense($syns, "sim");
 	
 	# put the synsets in a hash. this way we will avoid multiple
 	# copies of the same synset
@@ -475,8 +456,8 @@ sub sim
 	}
     }
     
-    # return the synsets in an array 
-    return(sort keys %simHash);
+    # return the synsets in an hash ref 
+    return(\%simHash);
 }
 
 # function to take a set of synsets and to return their entailment
@@ -486,29 +467,25 @@ sub enta
 {
     my $self = shift;
     my $wn = $self->{'wn'};
-    my @synsets = @_;
+    my ($synsetsh, $outprep) = @_;
     my %entailsHash = ();
     
     # check if this is a request for the input-output types of this
     # function
-    if ($#synsets == 0 && $synsets[0] eq '0')
-    {
-	# ah!
-	return(1, 1);
-    }
-    
-    my $i = 0;
-    for (; $i <= $#synsets; $i++)
+    return (1, 1) if(defined($outprep));
+
+    foreach my $syns (keys %{$synsetsh})
     {
 	# check if in word-pos-sense format
-	if ($synsets[$i] !~ /\#\w\#\d+/)
+        # TODO: Replace error message and exit with return error code.
+	if($syns !~ /\#\w\#\d+/)
 	{
-	    print STDERR "$synsets[$i] is not in WORD\#POS\#SENSE format!\n";
+	    print STDERR "$syns is not in WORD\#POS\#SENSE format!\n";
 	    exit;
 	}
 	
 	# get the entails synsets
-	my @entails = $wn->querySense($synsets[$i], "enta");
+	my @entails = $wn->querySense($syns, "enta");
 	
 	# put the entails synsets in a hash. this way we will avoid
 	# multiple copies of the same entails synset
@@ -519,8 +496,8 @@ sub enta
 	}
     }
     
-    # return the causs in an array 
-    return(sort keys %entailsHash);
+    # return the causs in an hash ref 
+    return(\%entailsHash);
 }
 
 # function to take a set of synsets and to return their cause
@@ -530,29 +507,25 @@ sub caus
 {
     my $self = shift;
     my $wn = $self->{'wn'};
-    my @synsets = @_;
+    my ($synsetsh, $outprep) = @_;
     my %causeHash = ();
     
     # check if this is a request for the input-output types of this
     # function
-    if ($#synsets == 0 && $synsets[0] eq '0')
-    {
-	# ah!
-	return(1, 1);
-    }
-    
-    my $i = 0;
-    for (; $i <= $#synsets; $i++)
+    return(1, 1) if(defined($outprep));
+
+    foreach my $syns (keys %{$synsetsh})
     {
 	# check if in word-pos-sense format
-	if ($synsets[$i] !~ /\#\w\#\d+/)
+        # TODO: Replace error message and exit with return error code.
+	if($syns !~ /\#\w\#\d+/)
 	{
-	    print STDERR "$synsets[$i] is not in WORD\#POS\#SENSE format!\n";
+	    print STDERR "$syns is not in WORD\#POS\#SENSE format!\n";
 	    exit;
 	}
 	
 	# get the cause synsets
-	my @cause = $wn->querySense($synsets[$i], "caus");
+	my @cause = $wn->querySense($syns, "caus");
 	
 	# put the cause synsets in a hash. this way we will avoid
 	# multiple copies of the same cause synset
@@ -563,8 +536,8 @@ sub caus
 	}
     }
     
-    # return the causs in an array 
-    return(sort keys %causeHash);
+    # return the causs in an hash ref 
+    return(\%causeHash);
 }
 
 # function to take a set of synsets and to return their participle
@@ -574,29 +547,25 @@ sub part
 {
     my $self = shift;
     my $wn = $self->{'wn'};
-    my @synsets = @_;
+    my ($synsetsh, $outprep) = @_;
     my %partHash = ();
     
     # check if this is a request for the input-output types of this
     # function
-    if ($#synsets == 0 && $synsets[0] eq '0')
-    {
-	# ah!
-	return(1, 1);
-    }
-    
-    my $i = 0;
-    for (; $i <= $#synsets; $i++)
+    return(1, 1) if(defined($outprep));
+
+    foreach my $syns (keys %{$synsetsh})
     {
 	# check if in word-pos-sense format
-	if ($synsets[$i] !~ /\#\w\#\d+/)
+        # TODO: Replace error message and exit with return error code.
+	if($syns !~ /\#\w\#\d+/)
 	{
-	    print STDERR "$synsets[$i] is not in WORD\#POS\#SENSE format!\n";
+	    print STDERR "$syns is not in WORD\#POS\#SENSE format!\n";
 	    exit;
 	}
 	
 	# get the part synsets
-	my @part = $wn->queryWord($synsets[$i], "part");
+	my @part = $wn->queryWord($syns, "part");
 	
 	# put the part synsets in a hash. this way we will avoid
 	# multiple copies of the same part synset
@@ -607,8 +576,8 @@ sub part
 	}
     }
     
-    # return the causs in an array 
-    return(sort keys %partHash);
+    # return the causs in an hash ref 
+    return(\%partHash);
 }
 
 # function to take a set of synsets and to return their pertainym
@@ -618,29 +587,25 @@ sub pert
 {
     my $self = shift;
     my $wn = $self->{'wn'};
-    my @synsets = @_;
+    my ($synsetsh, $outprep) = @_;
     my %pertHash = ();
     
     # check if this is a request for the input-output types of this
     # function
-    if ($#synsets == 0 && $synsets[0] eq '0')
-    {
-	# ah!
-	return(1, 1);
-    }
-    
-    my $i = 0;
-    for (; $i <= $#synsets; $i++)
+    return (1, 1) if(defined($outprep));
+
+    foreach my $syns (keys %{$synsetsh})
     {
 	# check if in word-pos-sense format
-	if ($synsets[$i] !~ /\#\w\#\d+/)
+        # TODO: Replace error message and exit with return error code.
+	if($syns !~ /\#\w\#\d+/)
 	{
-	    print STDERR "$synsets[$i] is not in WORD\#POS\#SENSE format!\n";
+	    print STDERR "$syns is not in WORD\#POS\#SENSE format!\n";
 	    exit;
 	}
 	
 	# get the pert synsets
-	my @pert = $wn->queryWord($synsets[$i], "pert");
+	my @pert = $wn->queryWord($syns, "pert");
 	
 	# put the pert synsets in a hash. this way we will avoid
 	# multiple copies of the same pert synset
@@ -651,8 +616,8 @@ sub pert
 	}
     }
     
-    # return the causs in an array 
-    return(sort keys %pertHash);
+    # return the causs in an hash ref 
+    return(\%pertHash);
 }
 
 # function to take a set of synsets and to return the concatenation of
@@ -662,39 +627,48 @@ sub glos
     my $self = shift;
     my $wn = $self->{'wn'};
     my $stemmer = $self->{'stemmer'};
-    my @synsets = @_;
+    my ($synsetsh, $outprep) = @_;
     my $returnString = "";
     
     # check if this is a request for the input-output types of this
     # function
-    if ($#synsets == 0 && $synsets[0] eq '0')
-    {
-	# ah!
-	return(1, 2);
-    }
-    
+    return (1, 2) if(defined($outprep));
+
+    my @synshkeys = keys %{$synsetsh};
     my $i = 0;
-    for (; $i <= $#synsets; $i++)
+    foreach my $syns (@synshkeys)
     {
 	# check if in word-pos-sense format
-	if ($synsets[$i] !~ /\#\w\#\d+/)
+        # TODO: Replace error message and exit with return error code.
+	if($syns !~ /\#\w\#\d+/)
 	{
-	    print STDERR "$synsets[$i] is not in WORD\#POS\#SENSE format!\n";
+	    print STDERR "$syns is not in WORD\#POS\#SENSE format!\n";
 	    exit;
 	}
 	
 	# get the glos
 	my $glosString;
-	($glosString) = $wn->querySense($synsets[$i], "glos");
+	($glosString) = $wn->querySense($syns, "glos");
 	
 	# regularize the glos
 	$glosString =~ s/\".*//;
-	$glosString =~ s/[^\w]/ /g;
+
+	# get rid of most punctuation
+	$glosString =~ tr/.;:,?!(){}\x22\x60\x24\x25\x40<>/ /;
+	# get rid of apostrophes not surrounded by word chars
+	$glosString =~ s/(?<!\w)\x27/ /g;
+	$glosString =~ s/\x27(?!\w)/ /g;
+	# remove dashes, but not hyphens
+	$glosString =~ s/--/ /g;
+
+	# this causes "plane's" to become "plane s"
+	# $glosString =~ s/[^\w]/ /g;
+
 	$glosString =~ s/\s+/ /g;
-	$glosString = lc($glosString);
+	$glosString = lc $glosString;
 
 	# stem the glos if asked for 
-	$glosString = $stemmer->stemString($glosString, 1) if ($self->{'stem'});
+	$glosString = $stemmer->stemString($glosString, 1) if($self->{stem});
 	
 	$glosString =~ s/^\s*/ /;
 	$glosString =~ s/\s*$/ /;
@@ -703,12 +677,13 @@ sub glos
 	$returnString .= $glosString;
 	
 	# put in boundary if more glosses coming!
-	if ($i < $#synsets) 
+	if($i < $#synshkeys) 
 	{ 
 	    my $boundary = sprintf("GGG%05dGGG", $self->{'glosBoundaryIndex'});
 	    $returnString .= $boundary;
 	    ($self->{'glosBoundaryIndex'})++;
 	}
+        $i++;
     }
     
     # and we are done!
@@ -722,36 +697,32 @@ sub example
     my $self = shift;
     my $wn = $self->{'wn'};
     my $stemmer = $self->{'stemmer'};    
-    my @synsets = @_;
+    my ($synsetsh, $outprep) = @_;
     my @exampleStrings = ();
     
     # check if this is a request for the input-output types of this
     # function
-    if ($#synsets == 0 && $synsets[0] eq '0')
-    {
-	# ah!
-	return(1, 2);
-    }
-    
-    # first get all the example strings into an array 
-    my $i = 0;
-    for (; $i <= $#synsets; $i++)
+    return (1, 2) if(defined($outprep));
+
+    # first get all the example strings into an array
+    foreach my $syns (keys %{$synsetsh})
     {
 	# check if in word-pos-sense format
-	if ($synsets[$i] !~ /\#\w\#\d+/)
+        # TODO: Replace error message and exit with return error code.
+	if($syns !~ /\#\w\#\d+/)
 	{
-	    print STDERR "$synsets[$i] is not in WORD\#POS\#SENSE format!\n";
+	    print STDERR "$syns is not in WORD\#POS\#SENSE format!\n";
 	    exit;
 	}
 	
 	# get the glos
 	my $exampleString;
-	($exampleString) = $wn->querySense($synsets[$i], "glos");
+	($exampleString) = $wn->querySense($syns, "glos");
 	
 	# check if this has any examples
-	if ($exampleString !~ /\"/) { next; }
+	if($exampleString !~ /\"/) {next;}
 	
-	while ($exampleString =~ /\"([^\"]*)\"/g)
+	while($exampleString =~ /\"([^\"]*)\"/g)
 	{
 	    push @exampleStrings, $1;
 	}
@@ -761,10 +732,21 @@ sub example
     # string. separate examples with the example boundary
     
     my $returnString = "";
-    foreach ($i = 0; $i <= $#exampleStrings; $i++)
+    my $i;
+    for ($i = 0; $i <= $#exampleStrings; $i++)
     {
 	# preprocess
-	$exampleStrings[$i] =~ s/[^\w]/ /g;
+
+	###
+	# get rid of most punctuation
+	$exampleStrings[$i] =~ tr/.;:,?!(){}\x22\x60\x24\x25\x40<>/ /;
+	# get rid of apostrophes not surrounded by word chars
+	$exampleStrings[$i] =~ s/(?<!\w)\x27/ /g;
+	$exampleStrings[$i] =~ s/\x27(?!\w)/ /g;
+	# remove dashes, but not hyphens
+	$exampleStrings[$i] =~ s/--/ /g;
+	###$exampleStrings[$i] =~ s/[^\w]/ /g;
+
 	$exampleStrings[$i] =~ s/\s+/ /g;
 	$exampleStrings[$i] =~ s/^\s*/ /;
 	$exampleStrings[$i] =~ s/\s*$/ /;
@@ -772,8 +754,8 @@ sub example
 	$exampleStrings[$i] = lc($exampleStrings[$i]);
 	
 	# stem if so required
-	$exampleStrings[$i] = $stemmer->stemString($exampleStrings[$i], 1) 
-	    if ($self->{'stem'});
+	$exampleStrings[$i] = $stemmer->stemString($exampleStrings[$i], 1)
+	    if($self->{'stem'});
 	
 	$exampleStrings[$i] =~ s/^\s*/ /;
 	$exampleStrings[$i] =~ s/\s*$/ /;
@@ -782,7 +764,7 @@ sub example
 	$returnString .= $exampleStrings[$i];
 		
 	# put in boundary if more examples coming!
-	if ($i < $#exampleStrings)
+	if($i < $#exampleStrings)
 	{ 
 	    my $boundary = sprintf("EEE%05dEEE", $self->{'exampleBoundaryIndex'});
 	    $returnString .= $boundary;
@@ -800,32 +782,26 @@ sub syns
 {
     my $self = shift;
     my $wn = $self->{'wn'};
-    my @synsets = @_;
+    my ($synsetsh, $outprep) = @_;
     my $returnString = "";
     
     # check if this is a request for the input-output types of this
     # function
-    if ($#synsets == 0 && $synsets[0] eq '0')
-    {
-	# ah!
-	return(1, 2);
-    }
-    
-    my $i = 0;
+    return (1, 2) if(defined($outprep));
 
     my %synonymHash = ();
-    
-    for (; $i <= $#synsets; $i++)
+    foreach my $syns (keys %{$synsetsh})
     {
 	# check if in word-pos-sense format
-	if ($synsets[$i] !~ /\#\w\#\d+/)
+        # TODO: Replace error message and exit with return error code.
+	if($syns !~ /\#\w\#\d+/)
 	{
-	    print STDERR "$synsets[$i] is not in WORD\#POS\#SENSE format!\n";
+	    print STDERR "$syns is not in WORD\#POS\#SENSE format!\n";
 	    exit;
 	}
 	
 	# get the words
-	my @synsetWords = $wn->querySense($synsets[$i], "syns");
+	my @synsetWords = $wn->querySense($syns, "syns");
 	
 	# for each word, remove the POS and SENSE, and put only the
 	# word in a hash
@@ -841,19 +817,19 @@ sub syns
     my @synonymArray = sort keys %synonymHash;
     
     # concatenate them, using the synonym boundary
-    for ($i = 0; $i <= $#synonymArray; $i++)
+    for(my $i = 0; $i <= $#synonymArray; $i++)
     {
 	$synonymArray[$i] =~ s/ /_/g;
 	$returnString .= " $synonymArray[$i] ";
 	
 	# put in boundary if more examples coming!
-	if ($i < $#synonymArray) 
+	if($i < $#synonymArray) 
 	{ 
-	    my $boundary = sprintf("SSS%05dSSS", $self->{'synonymBoundaryIndex'});
+	    my $boundary = sprintf("SSS%05dSSS", $self->{synonymBoundaryIndex});
 	    $returnString .= $boundary;
-	    ($self->{'synonymBoundaryIndex'})++;
+	    ($self->{synonymBoundaryIndex})++;
 	}
-    }	
+    }
     
     # and we are done!
     return($returnString);
@@ -864,41 +840,48 @@ sub syns
 sub glosexample
 {
     my $self = shift;
-    my $wn = $self->{'wn'};
-    my $stemmer = $self->{'stemmer'};
-    my @synsets = @_;
+    my $wn = $self->{wn};
+    my $stemmer = $self->{stemmer};
+    my ($synsetsh, $outprep) = @_;
     my $returnString = "";
     
     # check if this is a request for the input-output types of this
     # function
-    if ($#synsets == 0 && $synsets[0] eq '0')
-    {
-	# ah!
-	return(1, 2);
-    }
-    
-    my $i = 0;
-    for (; $i <= $#synsets; $i++)
+    return (1, 2) if(defined($outprep));
+
+    my @synshkeys = keys %{$synsetsh};
+    for(my $i = 0; $i < scalar(@synshkeys); $i++)
     {
 	# check if in word-pos-sense format
-	if ($synsets[$i] !~ /\#\w\#\d+/)
+        # TODO: Replace error message and exit with return error code.
+	if($synshkeys[$i] !~ /\#\w\#\d+/)
 	{
-	    print STDERR "$synsets[$i] is not in WORD\#POS\#SENSE format!\n";
+	    print STDERR "$synshkeys[$i] is not in WORD\#POS\#SENSE format!\n";
 	    exit;
 	}
 	
 	# get the glos
 	my $glosString;
-	($glosString) = $wn->querySense($synsets[$i], "glos");
+	($glosString) = $wn->querySense($synshkeys[$i], "glos");
 	
 	# regularize the glos
-	$glosString =~ s/\'//g;
-	$glosString =~ s/[^\w]/ /g;
+	###$glosString =~ s/\'//g;
+	###$glosString =~ s/[^\w]/ /g;
+
+	# get rid of most punctuation
+	$glosString =~ tr/.;:,?!(){}\x22\x60\x24\x25\x40<>/ /;
+	# get rid of apostrophes not surrounded by word chars
+	$glosString =~ s/(?<!\w)\x27/ /g;
+	$glosString =~ s/\x27(?!\w)/ /g;
+	# remove dashes, but not hyphens
+	$glosString =~ s/--/ /g;
+	###
+
 	$glosString =~ s/\s+/ /g;
-	$glosString = lc($glosString);
+	$glosString = lc $glosString;
 
 	# stem the glos if asked for 
-	$glosString = $stemmer->stemString($glosString, 1) if ($self->{'stem'});
+	$glosString = $stemmer->stemString($glosString, 1) if($self->{stem});
 	
 	$glosString =~ s/^\s*/ /;
 	$glosString =~ s/\s*$/ /;
@@ -907,11 +890,11 @@ sub glosexample
 	$returnString .= $glosString;
 	
 	# put in boundary if more glosses coming!
-	if ($i < $#synsets) 
+	if($i < $#synshkeys) 
 	{ 
-	    my $boundary = sprintf("XXX%05dXXX", $self->{'glosBoundaryIndex'});
+	    my $boundary = sprintf("XXX%05dXXX", $self->{glosBoundaryIndex});
 	    $returnString .= $boundary;
-	    ($self->{'glosBoundaryIndex'})++;
+	    ($self->{glosBoundaryIndex})++;
 	}
     }
     
